using System;
using System.Web;
using System.Net;
using SessionAwareWebSvcClient.SessionService;

namespace SessionAwareWebSvcClient
{
	/// <summary>
	/// Helper class that allows us to override the web request creation
	/// and modify request properties without editing the autogenerated web service proxy
	/// </summary>
	internal class MyRequestCreator: IWebRequestCreate
	{
		// Reference to the owner form
		private RequestData ReqData;

		public MyRequestCreator(RequestData reqData)
		{
			ReqData = reqData;
		}

		#region IWebRequestCreate Members

		public WebRequest Create(Uri uri)
		{
			ReqData.Request = new MyWebRequest(ReqData, uri);
			return ReqData.Request;
		}

		#endregion
	}

	/// <summary>
	/// Implements custom WebRequest that allows external access to the web response
	/// </summary>
	internal class MyWebRequest: WebRequest
	{
		private HttpWebRequest Request;
		private RequestData ReqData;
			
		public MyWebRequest(RequestData reqData, Uri uri)
		{
			ReqData = reqData;
			Request = HttpWebRequest.CreateDefault(uri) as HttpWebRequest;
			if ( ReqData.Cookie != null )
				Request.Headers.Add("Cookie", ReqData.Cookie);
			Request.AllowWriteStreamBuffering = true;
			//ReqData.Request = Request;
		}

		public override WebResponse GetResponse()
		{
			ReqData.Response = Request.GetResponse();
			// Get any cookies that are there. 
			// We do not expect anything except of the session cookie though
			ReqData.ExtractCookies();
			return ReqData.Response;
		}

		#region " Blind overrides - simply redirect calls to the instance of HttpWebRequest :
		public override void Abort()
		{
			Request.Abort ();
		}

		public override string ConnectionGroupName
		{
			get
			{
				return Request.ConnectionGroupName;
			}
			set
			{
				Request.ConnectionGroupName = value;
			}
		}

		public override long ContentLength
		{
			get
			{
				return Request.ContentLength;
			}
			set
			{
				Request.ContentLength = value;
			}
		}
		public override string ContentType
		{
			get
			{
				return Request.ContentType;
			}
			set
			{
				Request.ContentType = value;
			}
		}
		public override ICredentials Credentials
		{
			get
			{
				return Request.Credentials;
			}
			set
			{
				Request.Credentials = value;
			}
		}
		public override WebHeaderCollection Headers
		{
			get
			{
				return Request.Headers;
			}
			set
			{
				Request.Headers = value;
			}
		}
		public override string Method
		{
			get
			{
				return Request.Method;
			}
			set
			{
				Request.Method = value;
			}
		}
		public override System.IO.Stream GetRequestStream()
		{
			return Request.GetRequestStream ();
		}
		public override IWebProxy Proxy
		{
			get
			{
				return Request.Proxy;
			}
			set
			{
				Request.Proxy = value;
			}
		}
		public override Uri RequestUri
		{
			get
			{
				return Request.RequestUri;
			}
		}
		public override int Timeout
		{
			get
			{
				return Request.Timeout;
			}
			set
			{
				Request.Timeout = value;
			}
		}
		public override bool PreAuthenticate
		{
			get
			{
				return Request.PreAuthenticate;
			}
			set
			{
				Request.PreAuthenticate = value;
			}
		}
		public override IAsyncResult BeginGetRequestStream(AsyncCallback callback, object state)
		{
			return Request.BeginGetRequestStream (callback, state);
		}

		public override IAsyncResult BeginGetResponse(AsyncCallback callback, object state)
		{
			return Request.BeginGetResponse (callback, state);
		}
		public override System.IO.Stream EndGetRequestStream(IAsyncResult asyncResult)
		{
			return Request.EndGetRequestStream (asyncResult);
		}
		public override WebResponse EndGetResponse(IAsyncResult asyncResult)
		{
			return Request.EndGetResponse (asyncResult);
		}

		#endregion
	}

	/// <summary>
	/// Helper class to store request-related information
	/// </summary>
	public class RequestData
	{
		public WebRequest Request;
		public WebResponse Response;
		public string Cookie;

		public RequestData(string url)
		{
			WebRequest.RegisterPrefix(url, new MyRequestCreator(this));
			Cookie = ""; // No cookie initially
		}

		/// <summary>
		/// Get cookies from the web response. The response instance is "global" in a sense that we keep 
		/// a single object and update it when web service proxy requests it
		/// </summary>
		public void ExtractCookies()
		{
			// Only update the cookie when it's present
			if ( Response.Headers["Set-Cookie"] != null )
				Cookie = Response.Headers["Set-Cookie"];
		}
	}

}
